const fs = require('fs');
const path = require("path");
const jsonApi = require('really-relaxed-json');
const minimist = require('minimist');

const tag = new Date().getTime();

const MESSAGE_CATALOG_PATH = '../frontend/message_catalog.json';
const LANGS = ["cs"];
const INPUT_DIR = '../frontend/app/';
const OUTPUT_DIR = '../frontend/app/';
const TRANSLATIONS_DIR = '../frontend/app/translations/';
const DEFAULT_LANG = "en";

//const OUTPUT_DIR = './output/';
//const TRANSLATIONS_DIR = './output/translations/';



function emptyDelta() {
    return {
        changed: {},
        translated: {}
    };
}

function walkSync(dir, filelist) {
    var 
        files = fs.readdirSync(dir);
    
    filelist = filelist || [];
    files.forEach(function(file) {
        if (fs.statSync(dir + file).isDirectory()) {
            filelist = walkSync(dir + file + '/', filelist);
        } else if ("messages.js" == file) {
            filelist.push(dir  + file);
        }
    });
    return filelist;
};

function readFile(file) {
    let str = fs.readFileSync(file, 'utf8');

    var regex = /^\s*export\s*default\s*/;
    str = str.replace(regex, "");

    regex = /\}\s*;\s*$/;
    str = str.replace(regex, "}");

    str = str.trim();

    let data = str ? JSON.parse(jsonApi.toJson(str)) : null;
    if (!data) {
        return null;
    }

    return data;
}

function deltaFileName(file, lang) {
    let 
        dirname = path.dirname(file),
        basename = path.basename(file, ".js"),
        filename = dirname + "/" + basename + "-" + lang + ".json";
    return OUTPUT_DIR + filename;
}

function readDelta(deltaFilename) {
    if (!fs.existsSync(deltaFilename)) {
        return emptyDelta();
    }

    let str = fs.readFileSync(deltaFilename, 'utf8');
    if (!str) {
        return emptyDelta();
    }
    try {
        let delta = JSON.parse(jsonApi.toJson(str));
        return delta || emptyDelta();
    } catch (e) {
        console.log("Could not parse '" + deltaFilename + "'");
        return emptyDelta();
    }
}

function writeDelta(deltaFilename, data) {
    try {
        fs.mkdirSync(path.dirname(deltaFilename), { recursive: true });
    } catch (e) {
        //TODO
    }
    fs.writeFileSync(deltaFilename, JSON.stringify(data, null, 4));
}

function checkMetadata() {
    return !fs.existsSync(MESSAGE_CATALOG_PATH);
}

function readMetadata() {
    if (checkMetadata()) {
        return {};
    }
    let str = fs.readFileSync(MESSAGE_CATALOG_PATH, 'utf8');
    if (!str) {
        return {};
    }
    return JSON.parse(jsonApi.toJson(str));
}

function writeMetadata(metadata) {
    fs.writeFileSync(MESSAGE_CATALOG_PATH, JSON.stringify(metadata, null, 4));
}

function writeTranslationFile(list, lang) {
    try {
        fs.mkdirSync(TRANSLATIONS_DIR, { recursive: true });
    } catch (e) {
        //TODO
    }
    fs.writeFileSync(TRANSLATIONS_DIR + lang + ".js", "module.exports = " + JSON.stringify(list, null, 4));
}


function constructMetadata(files) {
    let shouldWrite = checkMetadata();
    let metadata = readMetadata(MESSAGE_CATALOG_PATH);
    let result = {};

    files.forEach((file) => {
        let data = readFile(file);
        if (!data) {
            return;
        }

        for (let key in data) {
            let message = data[key];
            let item = {
                id: message.id,
                defaultMessage: message.defaultMessage,
                langs: {},
                //version: tag,
                source: {
                    key,
                    file: path.relative(INPUT_DIR, file).replace(/\\/g, '/'),
                }
            };

            let m = metadata[item.id];
            if (m) {
                if (m.defaultMessage == item.defaultMessage) {
                    result[item.id] = m;
                } else {
                    result[item.id] = { ...item, langs: m.langs };
                }
            } else {
                result[item.id] = item;
            } 
        }

    });


    //if (shouldWrite) {
        writeMetadata(result);
    //}
    return result;
}


function pullDelta() {
    let metadata = constructMetadata(walkSync(INPUT_DIR));
    //writeMetadata(metadata);

    let files = {};
    for (let key in metadata) {
        let item = metadata[key];
        let fname = item.source.file;
        let file = files[fname];
        if (!file) {
            file = {};
            files[fname] = file;
        }
        file[item.source.key] = {
            id: item.id,
            defaultMessage: item.defaultMessage,
            //version: item.version
        };
    }

    for (let lang of LANGS) {
        for (let filename in files) {
            let 
                deltaFilename = deltaFileName(filename, lang),
                deltaFile = readDelta(deltaFilename),
                delta = emptyDelta(),
                file = files[filename];

            
            for (let key in file) {
                let item = file[key];

                let m = metadata[item.id];
                let l = m.langs[lang];

                if (l) {
                    let df = deltaFile.translated[key];

                    if (df && (df.defaultMessage == m.defaultMessage || !df.defaultMessage)) {
                        delta.translated[key] = df;

                    } else {
                        delta.changed[key] = {
                            id: item.id,
                            message: l || item.defaultMessage,
                            defaultMessage: item.defaultMessage,
                            //version: tag
                        }
                    }
                } else {
                    //not in metadata, check whether it is already translated...?
                    l = deltaFile.translated[key];
                    if (l) {
                        delta.translated[key] = l;
                    } else {
                        delta.changed[key] = {
                            id: item.id,
                            message: item.defaultMessage,
                            defaultMessage: item.defaultMessage,
                            //version: m.version
                        }
                    }
                }

                //deltaFile[key] = file[key];
            }
            
            writeDelta(deltaFilename, delta);
        }
    }

}

function pushDelta() {
    let metadata = readMetadata(MESSAGE_CATALOG_PATH);

    //construct paths
    let paths = {};
    for (let mk in metadata) {
        let m = metadata[mk];
        if (m.source && m.source.file) {
            paths[m.source.file] = true;
        }
    }

    //read delta files as well
    let files = walkSync(INPUT_DIR);
    files = files.map(file => path.relative(INPUT_DIR, file).replace(/\\/g, '/'));
    files.forEach(file => paths[file] = true);

    let outputLangs = {};

    for (let lang of LANGS) {
        outputLangs[lang] = [];
        let ids = {};

        for (let p in paths) {
            let dfp = deltaFileName(p, lang);
            let delta = readDelta(dfp);
            for (let key in delta.translated) {
                let l = delta.translated[key];
                let m = metadata[l.id];
                if (!m) {
                    m = {
                        id: l.id,
                        defaultMessage: l.defaultMessage,
                        langs: {},
                        //version: tag,
                        source: {
                            key,
                            file: p,
                        }
                    };

                    metadata[l.id] = m;
                }

                delete m.version;
                ids[l.id] = true;
                m.langs[lang] = l.message;
                outputLangs[lang].push({
                    id: l.id,
                    defaultMessage: l.message
                });
            }
        }

        for (let id in metadata) {
            let m = metadata[id];
            if (m.langs[lang] && !ids[id]) {
                outputLangs[lang].push({
                    id: id,
                    defaultMessage: m.langs[lang]
                });
            }
        }

    }

    //write translations
    for (let lang of LANGS) {
        writeTranslationFile(outputLangs[lang], lang);
    }

    outputLangs[DEFAULT_LANG] = [];
    for (let key in metadata) {
        let m = metadata[key];
        outputLangs[DEFAULT_LANG].push({
            id: m.id,
            defaultMessage: m.defaultMessage
        });
    }    
    writeTranslationFile(outputLangs[DEFAULT_LANG], DEFAULT_LANG);


    writeMetadata(metadata);
}




let args = minimist(process.argv.slice(2));
if (!args._.length) {
    process.exit(0);
}

let action = args._[0];
if (action == "pull") {
    pullDelta();
} else if (action == "push") {
    pushDelta();
}

